import { createDefaultMetadata } from "./default-metadata";
import { resolveOpenGraph } from "./resolve-opengraph";
import { mergeTitle } from "./resolve-title";
const viewPortKeys = {
    width: "width",
    height: "height",
    initialScale: "initial-scale",
    minimumScale: "minimum-scale",
    maximumScale: "maximum-scale",
    viewportFit: "viewport-fit"
};
// Merge the source metadata into the resolved target metadata.
function merge(target, source, templateStrings) {
    for(const key_ in source){
        const key = key_;
        switch(key){
            case "other":
                {
                    Object.assign(target.other, source.other);
                    break;
                }
            case "title":
                {
                    if (source.title) {
                        target.title = source.title;
                        mergeTitle(target, templateStrings.title);
                    }
                    break;
                }
            case "openGraph":
                {
                    if (typeof source.openGraph !== "undefined") {
                        target.openGraph = resolveOpenGraph(source.openGraph);
                        if (source.openGraph) {
                            mergeTitle(target.openGraph, templateStrings.openGraph);
                        }
                    } else {
                        target.openGraph = null;
                    }
                    break;
                }
            case "twitter":
                {
                    if (source.twitter) {
                        target.twitter = source.twitter;
                        mergeTitle(target.twitter, templateStrings.twitter);
                    } else {
                        target.twitter = null;
                    }
                    break;
                }
            case "viewport":
                {
                    let content = null;
                    const { viewport  } = source;
                    if (typeof viewport === "string") {
                        content = viewport;
                    } else if (viewport) {
                        content = "";
                        for(const viewportKey_ in viewPortKeys){
                            const viewportKey = viewportKey_;
                            if (viewport[viewportKey]) {
                                if (content) content += ", ";
                                content += `${viewPortKeys[viewportKey]}=${viewport[viewportKey]}`;
                            }
                        }
                    }
                    target.viewport = content;
                    break;
                }
            default:
                {
                    // TODO: Make sure the type is correct.
                    // @ts-ignore
                    target[key] = source[key];
                    break;
                }
        }
    }
}
export async function resolveMetadata(metadataItems) {
    const resolvedMetadata = createDefaultMetadata();
    let committedTitleTemplate = null;
    let committedOpenGraphTitleTemplate = null;
    let committedTwitterTitleTemplate = null;
    let lastLayer = 0;
    // from root layout to page metadata
    for(let i = 0; i < metadataItems.length; i++){
        const item = metadataItems[i];
        const isLayout = item.type === "layout";
        const isPage = item.type === "page";
        if (isLayout || isPage) {
            let layerMod = await item.mod();
            // Layer is a client component, we just skip it. It can't have metadata
            // exported. Note that during our SWC transpilation, it should check if
            // the exports are valid and give specific error messages.
            if ("$$typeof" in layerMod && layerMod.$$typeof === Symbol.for("react.module.reference")) {
                continue;
            }
            if (layerMod.metadata && layerMod.generateMetadata) {
                throw new Error(`A ${item.type} is exporting both metadata and generateMetadata which is not supported. If all of the metadata you want to associate to this ${item.type} is static use the metadata export, otherwise use generateMetadata. File: ` + item.path);
            }
            // If we resolved all items in this layer, commit the stashed titles.
            if (item.layer >= lastLayer) {
                var ref, ref1, ref2, ref3, ref4;
                committedTitleTemplate = ((ref = resolvedMetadata.title) == null ? void 0 : ref.template) || null;
                committedOpenGraphTitleTemplate = ((ref1 = resolvedMetadata.openGraph) == null ? void 0 : (ref2 = ref1.title) == null ? void 0 : ref2.template) || null;
                committedTwitterTitleTemplate = ((ref3 = resolvedMetadata.twitter) == null ? void 0 : (ref4 = ref3.title) == null ? void 0 : ref4.template) || null;
                lastLayer = item.layer;
            }
            if (layerMod.metadata) {
                merge(resolvedMetadata, layerMod.metadata, {
                    title: committedTitleTemplate,
                    openGraph: committedOpenGraphTitleTemplate,
                    twitter: committedTwitterTitleTemplate
                });
            } else if (layerMod.generateMetadata) {
                merge(resolvedMetadata, await layerMod.generateMetadata(// TODO: Rewrite this to pass correct params and resolving metadata value.
                {}, Promise.resolve(resolvedMetadata)), {
                    title: committedTitleTemplate,
                    openGraph: committedOpenGraphTitleTemplate,
                    twitter: committedTwitterTitleTemplate
                });
            }
        }
    }
    return resolvedMetadata;
}
// TODO: Implement this function.
export async function resolveFileBasedMetadataForLoader(_layer, _dir) {
    let metadataCode = "";
    // const files = await fs.readdir(path.normalize(dir))
    // for (const file of files) {
    //   // TODO: Get a full list and filter out directories.
    //   if (file === 'icon.svg') {
    //     metadataCode += `{
    //       type: 'icon',
    //       layer: ${layer},
    //       path: ${JSON.stringify(path.join(dir, file))},
    //     },`
    //   } else if (file === 'icon.jsx') {
    //     metadataCode += `{
    //       type: 'icon',
    //       layer: ${layer},
    //       mod: () => import(/* webpackMode: "eager" */ ${JSON.stringify(
    //         path.join(dir, file)
    //       )}),
    //       path: ${JSON.stringify(path.join(dir, file))},
    //     },`
    //   }
    // }
    return metadataCode;
}

//# sourceMappingURL=resolve-metadata.js.map